#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <err.h>

#include "dns.h"
#include "dns/db/file.h"

int main(void) {
	struct dns_message *m;
	struct dns_message *n;
	unsigned char pkt[4096];
	size_t len;
	struct dns_record *rr;
	enum dns_errno err;

	/* #!/usr/bin/perl
	 *
	 * use Net::DNS;
	 *
	 * my $res	= Net::DNS::Resolver->new;
	 * my $so	= $res->bgsend(shift @ARGV || "google.com");
	 *
	 * print $res->bgread($so)->data;
	 */
	assert(0 < (len = fread(pkt, 1, sizeof pkt, stdin)));

	if (0 == (m = dns_m_parse(pkt, len, &err)))
		errx(EXIT_FAILURE, "error:%d", err);

	fprintf(stderr, "id:%hu q:%hu/%hu ans:%hu/%hu add:%hu/%hu auth:%hu/%hu\n", m->qid, m->qd.found, m->qd.count, m->an.found, m->an.count, m->ns.found, m->ns.count, m->ar.found, m->ar.count);

	n	= dns_m_open(&err);

	DNS_M_FOREACH(rr, m, &m->all, DNS_DN_ANY, DNS_RR_C(IN), DNS_RR_T(A, TXT, NS, MX)) {
		char name[DNS_DN_MAXSIZE];
		union { unsigned char any[512]; struct dns_rr_a a; struct dns_rr_txt txt; struct dns_rr_ns ns; struct dns_rr_mx mx; } rdata;

		if (-1 == dns_rr_expand(name, sizeof name, &rdata, sizeof rdata, rr->dn.base, m->data, m->length, (rr->qr)? DNS_RR_QR : 0, &err))
			errx(EXIT_FAILURE, "failed to expand name (%d/%d): %s", rr->class, rr->type, dns_errlist[err]);

		switch (rr->type) {
#if 0
		case DNS_RR_T_TXT:
			if (0 != dns_m_txt_parse(m, &txt, rr))
				errx(EXIT_FAILURE, "failed to parse TXT: %s", dns_errlist[dns_m_errno(m)]);

			dns_m_txt_print(m, rdata, sizeof rdata, &txt, DNS_M_TXT_CONCAT);

			fprintf(stderr, "%s type:%d  class:%d  ttl:%d  rdata:(%u) '%s'\n", name, rr->type, rr->class, rr->ttl, txt.count, rdata);

			break;
#endif
		default:
			fprintf(stderr, "(@%d) %s type:%d  class:%d  ttl:%d  rdata:%d\n", rr->dn.base, name, rr->type, rr->class, rr->ttl, rr->rd.length);
		}

		if (!dns_m_push(n, (rr->qr)? &n->qd : &n->an, name, -1, rr->class, rr->type, rr->ttl, &rdata, -1, 0, &err))
			errx(EXIT_FAILURE, "failed to copy RR: %s", dns_errlist[err]);
	}

	fwrite(n->data, 1, n->length, stdout);

#if 0
	struct dns_database *db	= dns_db_file_open(fopen("/etc/hosts", "r"), 0);

	unsigned char dn[256];

	fprintf(stderr, "www.google.com => ");

	if (-1 != (len = dns_m_dn_comp(m, dn, sizeof dn, "www.google.com", -1, 0))) {
		for (unsigned i = 0; i < len; i++) {
			fprintf(stderr, "%.2x ", dn[i]);
		}
		fprintf(stderr, "\n");
	} else
		fprintf(stderr, "no comp\n");
#endif

	return 0;
} /* main() */
