/* ==========================================================================
 * dns/inline.h - Inline macros & routines.
 * --------------------------------------------------------------------------
 * Copyright (c) 2007  William Ahern
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to permit
 * persons to whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 * USE OR OTHER DEALINGS IN THE SOFTWARE.
 * ==========================================================================
 */
#ifndef DNS_INLINE_H
#define DNS_INLINE_H

#include <stdbool.h>	/* bool */

#include <limits.h>	/* CHAR_BIT */

#include <sys/param.h>	/* howmany roundup rounddown MIN MAX */


/*
 * BSD'ish macros
 */
#ifndef howmany
#define howmany(x, y)	(((x) + ((y) - 1)) / (y))
#endif

#ifndef roundup
#define roundup(x, y)	((((x) + ((y) - 1)) / (y)) * (y))
#endif

#ifndef rounddown
#define rounddown(x, y)	((x) - ((x) % (y)))
#endif

#ifndef arraylen
#define arraylen(a)	(sizeof (a) / sizeof (a)[0])
#endif

#ifndef MIN
#define MIN(a, b)	(((a) < (b))? (a) : (b))
#endif


/*
 * Macros for bit arrays and other sundry integral tasks.
 */
#define DNS__INDEXOF(b, type)		((b) / (sizeof (type) * CHAR_BIT))
#define DNS__VALUEOF(b, type)		(1UL << ((b) % (sizeof (type) * CHAR_BIT)))
#define DNS__HOWMANY(x, y)		(((x) + ((y) - 1)) / (y))
#define DNS__ROUNDUP(x, y)		((((x) + ((y) - 1)) / (y)) * (y))
#define DNS__ROUNDDN(x, y)		((x) - ((x) % (y)))
#define DNS__ARRAYLEN(a)		(sizeof (a) / sizeof (a)[0])
#define DNS__ARRAYBITS(a)		(sizeof (a) * CHAR_BIT)
#define DNS__MIN(a, b)			(((a) < (b))? (a) : (b))


/*
 * Standard token pasting stuff.
 */
#define DNS__PASTE(x, y)	x##y
#define DNS__XPASTE(x, y)	DNS__PASTE(x, y)


/*
 * The DNS__PP_NARG macro evaluates to the number of arguments that have
 * been passed to it.
 *
 * Laurent Deniau, "__VA_NARG__," 17 January 2006, <comp.std.c> (29 November 2007).
 */
#define DNS__PP_NARG(...)	DNS__PP_NARG_(__VA_ARGS__,DNS__PP_RSEQ_N())
#define DNS__PP_NARG_(...)	DNS__PP_ARG_N(__VA_ARGS__)

#define DNS__PP_ARG_N( \
	_1, _2, _3, _4, _5, _6, _7, _8, _9,_10,  \
	_11,_12,_13,_14,_15,_16,_17,_18,_19,_20, \
	_21,_22,_23,_24,_25,_26,_27,_28,_29,_30, \
	_31,_32,_33,_34,_35,_36,_37,_38,_39,_40, \
	_41,_42,_43,_44,_45,_46,_47,_48,_49,_50, \
	_51,_52,_53,_54,_55,_56,_57,_58,_59,_60, \
	_61,_62,_63,N,...) N

#define DNS__PP_RSEQ_N() \
	63,62,61,60,                   \
	59,58,57,56,55,54,53,52,51,50, \
	49,48,47,46,45,44,43,42,41,40, \
	39,38,37,36,35,34,33,32,31,30, \
	29,28,27,26,25,24,23,22,21,20, \
	19,18,17,16,15,14,13,12,11,10, \
	9,8,7,6,5,4,3,2,1,0 


/*
 * Select leading N items from list.
 */
#define DNS__PP_ARGV(N, ...)	DNS__PASTE(DNS__PP_ARGV, N)(__VA_ARGS__)
#define DNS__PP_ARGV1(_00,...) \
	_00
#define DNS__PP_ARGV2(_00,_01,...) \
	_00,_01
#define DNS__PP_ARGV3(_00,_01,_02,...) \
	_00,_01,_02
#define DNS__PP_ARGV4(_00,_01,_02,_03,...) \
	_00,_01,_02,_03
#define DNS__PP_ARGV5(_00,_01,_02,_03,_04,...) \
	_00,_01,_02,_03,_04
#define DNS__PP_ARGV6(_00,_01,_02,_03,_04,_05,...) \
	_00,_01,_02,_03,_04,_05
#define DNS__PP_ARGV7(_00,_01,_02,_03,_04,_05,_06,...) \
	_00,_01,_02,_03,_04,_05,_06
#define DNS__PP_ARGV8(_00,_01,_02,_03,_04,_05,_06,_07,...) \
	_00,_01,_02,_03,_04,_05,_06,_07
#define DNS__PP_ARGV9(_00,_01,_02,_03,_04,_05,_06,_07,_08,...) \
	_00,_01,_02,_03,_04,_05,_06,_07,_08
#define DNS__PP_ARGV10(_00,_01,_02,_03,_04,_05,_06,_07,_08,_09,...) \
	_00,_01,_02,_03,_04,_05,_06,_07,_08,_09
#define DNS__PP_ARGV11(_00,_01,_02,_03,_04,_05,_06,_07,_08,_09,_0a,...) \
	_00,_01,_02,_03,_04,_05,_06,_07,_08,_09,_0a
#define DNS__PP_ARGV12(_00,_01,_02,_03,_04,_05,_06,_07,_08,_09,_0a,_0b,...) \
	_00,_01,_02,_03,_04,_05,_06,_07,_08,_09,_0a,_0b
#define DNS__PP_ARGV13(_00,_01,_02,_03,_04,_05,_06,_07,_08,_09,_0a,_0b,_0c,...) \
	_00,_01,_02,_03,_04,_05,_06,_07,_08,_09,_0a,_0b,_0c
#define DNS__PP_ARGV14(_00,_01,_02,_03,_04,_05,_06,_07,_08,_09,_0a,_0b,_0c,_0d,...) \
	_00,_01,_02,_03,_04,_05,_06,_07,_08,_09,_0a,_0b,_0c,_0d
#define DNS__PP_ARGV15(_00,_01,_02,_03,_04,_05,_06,_07,_08,_09,_0a,_0b,_0c,_0d,_0e,...) \
	_00,_01,_02,_03,_04,_05,_06,_07,_08,_09,_0a,_0b,_0c,_0d,_0e
#define DNS__PP_ARGV16(_00,_01,_02,_03,_04,_05,_06,_07,_08,_09,_0a,_0b,_0c,_0d,_0e,_0f,...) \
	_00,_01,_02,_03,_04,_05,_06,_07,_08,_09,_0a,_0b,_0c,_0d,_0e,_0f
	

/*
 * Apply function-like macro F to each item in argument list.
 */
#define DNS__PP_MAP16(F,_00,_01,_02,_03,_04,_05,_06,_07,_08,_09,_0a,_0b,_0c,_0d,_0e,_0f,...) \
	F(_00),F(_01),F(_02),F(_03),F(_04),F(_05),F(_06),F(_07), \
	F(_08),F(_09),F(_0a),F(_0b),F(_0c),F(_0d),F(_0e),F(_0f)

#define DNS__PP_MAP_(F,...) DNS__PP_MAP16(F,__VA_ARGS__)
#define DNS__PP_MAP(F,...) DNS__PP_ARGV(DNS__PP_NARG(__VA_ARGS__),DNS__PP_MAP_(F,__VA_ARGS__,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0))


/*
 * Enumeration sets, because bit fields don't cut it when you have a 16-bit
 * space like with RFC 1035 type and class identifiers.
  */
#define DNS_ENUMSET_MAXINDEX	255
#define DNS_ENUMSET_MAXCOUNT	63	/* NB: Limited by DNS__PP_NARG */

typedef unsigned long dns__enum_index_t;
typedef unsigned int dns_enum_t;

struct dns_enumset {
	dns__enum_index_t index[DNS__HOWMANY(DNS_ENUMSET_MAXINDEX, sizeof (dns__enum_index_t) * CHAR_BIT)];
	dns_enum_t list[DNS_ENUMSET_MAXCOUNT];
	unsigned int count;
}; /* struct dns_enumset */

#define DNS_ENUMSET_INITIALIZER	{ .index = { 0 }, .count = 0 }
#define DNS_ENUMSET_ANY		(struct dns_enumset *)0
#define DNS_ENUMSET_EMPTY	&(struct dns_enumset)DNS_ENUMSET_INITIALIZER
#define DNS_ENUMSET_OF(...)	dns_enumset_addn(DNS_ENUMSET_EMPTY, DNS__PP_NARG(__VA_ARGS__), (dns_enum_t[]){ __VA_ARGS__ })

static inline struct dns_enumset *dns_enumset_add(struct dns_enumset *E, dns_enum_t e) {
	if (e < DNS__ARRAYBITS(E->index))
		E->index[DNS__INDEXOF(e, dns__enum_index_t)] |= DNS__VALUEOF(e, dns__enum_index_t);

	if (E->count < DNS__ARRAYLEN(E->list))
		E->list[E->count++]	= e;

	return E;	
} /* dns_enumset_add() */


static inline struct dns_enumset *dns_enumset_addn(struct dns_enumset *E, unsigned int count, const dns_enum_t *list) {
	for (unsigned int i = 0; i < DNS__MIN(count, DNS_ENUMSET_MAXCOUNT); i++)
		dns_enumset_add(E, list[i]);

	return E;	
} /* dns_enumset_addn() */


static inline bool dns_enumset_has(const struct dns_enumset *E, dns_enum_t e) {
	if (E == DNS_ENUMSET_ANY)
		return 1;

	if (e < DNS__ARRAYBITS(E->index))
		return E->index[DNS__INDEXOF(e, dns__enum_index_t)] & DNS__VALUEOF(e, dns__enum_index_t);

	for (unsigned int i = 0; i < E->count; i++) {
		if (E->list[i] == e)
			return 1;
	}

	return 0;
} /* dns_enumset_has() */



/*
 * Bit packing implementation for reading and writing bit field values in
 * octet buffers. Both `base' and `len' are bit units expressing the
 * location of an unsigned integer hidden in the murky depths of the buffer.
 *
 * XXX: No bounds checking. The conditional was preventing GCC (3.3.5 AMD64)
 * from inlining. When inlined with constant `base' and `len' the code is
 * almost the same complexity as manual parsing; otherwise its a horrendous
 * glob of operations wrapped in a functional call. The improvement in
 * readability and abstraction is worthwhile, IMHO.
 *
 * NB: If the stride (number of octets the field spans) is greater than the
 * size of the bit source/sink integer than things break. For a 32-bit
 * integer any field over 25 bits should be octet aligned, or at least not
 * so offset that the stride is too long. A 26-bit field could span 5 octets
 * if the first and last bits were the lsb and msb of the first and last
 * spanned octets, respectively.
 */
#define DNS__PACK_STRIDE(base, len)	DNS__HOWMANY((base + len) - DNS__ROUNDDN(base, 8), 8)
#define DNS__PACK_OFFSET(base, len)	(0x07 & (8 - (((base) + (len)) % 8)))
#define DNS__PACK_MASK(base, len)	(1UL << ((len) - 1)) | ((1UL << ((len) - 1)) - 1)

typedef unsigned int dns_intmax_t;

static inline dns_intmax_t dns_unpack(unsigned base, unsigned len, const unsigned char *src) {
	dns_intmax_t val	= 0;	/* Use long long for unaligned 32-bit values; 26-bit value could span 5 octets. */
	const unsigned shift	= DNS__PACK_OFFSET(base, len);
	const dns_intmax_t mask	= DNS__PACK_MASK(base, len);

	src	+= base / 8;

	switch (DNS__PACK_STRIDE(base, len)) {
	case 4:
		val	|= (0xff & *src++) << 24;
	case 3:
		val	|= (0xff & *src++) << 16;
	case 2:
		val	|= (0xff & *src++) << 8;
	case 1:
		val	|= (0xff & *src++) << 0;
	}

	return mask & (val >> shift);
} /* dns_unpack() */


static inline dns_intmax_t dns_pack(unsigned char *dst, unsigned base, unsigned len, dns_intmax_t val) {
	const unsigned shift	= DNS__PACK_OFFSET(base, len);
	const dns_intmax_t mask	= ~(DNS__PACK_MASK(base, len) << shift);

	dst	+= base / 8;
	val	<<= shift;

	switch (DNS__PACK_STRIDE(base, len)) {
	case 4:
		*dst	= (*dst & (mask >> 24)) | (val >> 24);
		dst++;
	case 3:
		*dst	= (*dst & (mask >> 16)) | (val >> 16);
		dst++;
	case 2:
		*dst	= (*dst & (mask >> 8)) | (val >> 8);
		dst++;
	case 1:
		*dst	= (*dst & (mask >> 0)) | (val >> 0);
		dst++;
	}

	return val >> shift;
} /* dns_pack() */


#endif /* DNS_INLINE_H */
