/* ==========================================================================
 * dns/rr.h - DNS library.
 * --------------------------------------------------------------------------
 * Copyright (c) 2007  William Ahern
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to permit
 * persons to whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 * USE OR OTHER DEALINGS IN THE SOFTWARE.
 * ==========================================================================
 */
#ifndef DNS_RR_H
#define DNS_RR_H


enum dns_rr_class {
	DNS_RR_C_IN	= 1,
	DNS_RR_C_CHAOS	= 3,
	DNS_RR_C_HS	= 4,
	DNS_RR_C_ANY	= 255
}; /* enum dns_rr_class */


#define DNS_RR_C_FQN(c)		DNS__PASTE(DNS_RR_C_, c)
#define DNS_RR_C(...)		DNS_ENUMSET_OF(DNS__PP_MAP(DNS_RR_C_FQN, __VA_ARGS__))
#define DNS_RR_C_ANY()		DNS_ENUMSET_ANY


enum dns_rr_type {
	DNS_RR_T_A	= 1,
	DNS_RR_T_NS	= 2,
	DNS_RR_T_CNAME	= 5,
	DNS_RR_T_SOA	= 6,
	DNS_RR_T_PTR	= 12,
	DNS_RR_T_MX	= 15,
	DNS_RR_T_TXT	= 16,
	DNS_RR_T_AAAA	= 28,
	DNS_RR_T_SRV	= 33,
	DNS_RR_T_ANY	= 255,
}; /* enum dns_rr_type */


#define DNS_RR_T_FQN(c)		DNS__PASTE(DNS_RR_T_, c)
#define DNS_RR_T(...)		DNS_ENUMSET_OF(DNS__PP_MAP(DNS_RR_T_FQN, __VA_ARGS__))
#define DNS_RR_T_ANY()		DNS_ENUMSET_ANY


struct dns_dict;

typedef void *(*dns_rr_parse_fn)(void *, size_t, unsigned short, unsigned short, const void *, size_t, unsigned int, enum dns_errno *);
typedef size_t (*dns_rr_comp_fn)(void *, size_t, const void *, struct dns_dict *, const void *, size_t, enum dns_errno *);

struct dns_rr_parser {
	dns_rr_parse_fn parse;
	dns_rr_comp_fn comp;
}; /* struct dns_rr_parser */

const struct dns_rr_parser *dns_rr_addparser(enum dns_rr_class, enum dns_rr_type, dns_rr_parse_fn, dns_rr_comp_fn, enum dns_errno *);
const struct dns_rr_parser *dns_rr_getparser(enum dns_rr_class, enum dns_rr_type, enum dns_errno *);

struct dns_record *dns_rr_parse(struct dns_record *, unsigned short, const void *, size_t, unsigned int, enum dns_errno *);
size_t dns_rr_expand(void *, size_t, void *, size_t, unsigned short, const void *, size_t, unsigned int, enum dns_errno *);
size_t dns_rr_comp(void *, size_t, const void *, size_t, enum dns_rr_class, enum dns_rr_type, unsigned int, const void *, struct dns_dict *, const void *, size_t, enum dns_errno *);

#endif /* DNS_RR_H */
