/* ==========================================================================
 * dns/rr/aaaa.c - DNS library.
 * --------------------------------------------------------------------------
 * Copyright (c) 2007  William Ahern
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to permit
 * persons to whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 * USE OR OTHER DEALINGS IN THE SOFTWARE.
 * ==========================================================================
 */
#include <string.h>	/* memcpy(3) */


void *dns_rr_aaaa_parse(void *rr, size_t rrsiz, unsigned short rdp, unsigned short rdlen, const void *data, size_t datalen, unsigned int flags, enum dns_errno *err) {
	struct dns_rr_aaaa *a6	= rr;

	if (rrsiz < sizeof *a6 || rdlen != 16)
		return (*err = DNS_EINVALID), (void *)0;

	memcpy(a6->addr, (char *)data + rdp, 16);

	return a6;
} /* dns_rr_aaaa_parse() */


size_t dns_rr_aaaa_comp(void *dst, size_t dstsiz, const void *rr, struct dns_dict *dict, const void *data, size_t datalen, enum dns_errno *err) {
	const struct dns_rr_aaaa *a6	= rr;

	memcpy(dst, a6->addr, MIN(dstsiz, 16));

	return 16;
} /* dns_rr_aaaa_comp() */

