/* ==========================================================================
 * dns/rr/mx.c - DNS library.
 * --------------------------------------------------------------------------
 * Copyright (c) 2007  William Ahern
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to permit
 * persons to whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 * USE OR OTHER DEALINGS IN THE SOFTWARE.
 * ==========================================================================
 */


void *dns_rr_mx_parse(void *rr, size_t rrsiz, unsigned short rdp, unsigned short rdlen, const void *data, size_t datalen, unsigned int flags, enum dns_errno *err) {
	struct dns_rr_mx *mx	= rr;
	size_t len;

	if (rrsiz < sizeof *mx)
		goto invalid;

	if (rdlen < 3)
		goto truncat;

	mx->preference	= dns_unpack(0, 16, (unsigned char *)data + rdp);

	if (-1 == (len = dns_dn_expand(mx->host, sizeof mx->host, rdp + 2, data, datalen, flags, err)))
		return 0;

	mx->hostlen	= len;

	return mx;
truncat:
	*err	= DNS_ETRUNCATED;

	return 0;
invalid:
	*err	= DNS_EINVALID;

	return 0;
} /* dns_rr_mx_parse() */


size_t dns_rr_mx_comp(void *dst, size_t dstsiz, const void *rr, struct dns_dict *dict, const void *data, size_t datalen, enum dns_errno *err) {
	const struct dns_rr_mx *mx	= rr;
	size_t len;
	struct dns_name dn;

	if (dstsiz >= 2)
		dns_pack(dst, 0, 16, mx->preference);
		
	if (-1 == (len = dns_dn_comp((char *)dst + 2, (dstsiz > 2)? dstsiz - 2 : 0, mx->host, mx->hostlen, dict, data, datalen, err)))
		return -1;

	(void)dns_dict_appendbuf(dict, 2, dst, dstsiz, data, datalen);

	return 2 + len;
} /* dns_rr_mx_comp() */

