/* ==========================================================================
 * dns/rr/txt.h - DNS library.
 * --------------------------------------------------------------------------
 * Copyright (c) 2007  William Ahern
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to permit
 * persons to whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 * USE OR OTHER DEALINGS IN THE SOFTWARE.
 * ==========================================================================
 */
#include <stddef.h>	/* offsetof */

#ifndef DNS_RR_TXT_H
#define DNS_RR_TXT_H


#ifndef DNS_RR_TXT_MINDATA
#define DNS_RR_TXT_MINDATA	2	/* NB: Mutable. */
#endif

#define DNS_RR_TXT_MAXDATA	65535	/* Maximum data strings (or total length) */

/* NB: Size of txt object which can hold N data strings. */
#define DNS_RR_TXT_SIZEOF(N)	(offsetof(struct dns_rr_txt, data) + (sizeof (((struct dns_rr_txt *)0)->data) * (N)))

/* NB: Maximum number of data strings a txt object of size Z can hold. */
#define DNS_RR_TXT_LIMITOF(Z)	(((Z) - offsetof(struct dns_rr_txt, data)) / sizeof (((struct dns_rr_txt *)0)->data))

#define DNS_RR_TXT_DATA_MINSIZE	0
#define DNS_RR_TXT_DATA_MAXSIZE	255


struct dns_rr_txt {
	struct {
		unsigned char count;
		unsigned short length;
	} tc;

	unsigned char count;
	unsigned short length;

	struct dns_rr_txt_data {
		unsigned char length;
		unsigned char string[DNS_RR_TXT_DATA_MAXSIZE + 1];
	} data[DNS_RR_TXT_MINDATA];
}; /* struct dns_rr_txt */


void *dns_rr_txt_parse(void *, size_t, unsigned short, unsigned short, const void *, size_t, unsigned int, enum dns_errno *);
size_t dns_rr_txt_comp(void *, size_t, const void *, struct dns_dict *, const void *, size_t, enum dns_errno *);

size_t dns_rr_txt_concat(void *, size_t, struct dns_rr_txt *, unsigned char, unsigned char);
struct dns_rr_txt *dns_rr_txt_printf(void *, size_t, const char *, ...);


#endif /* DNS_RR_TXT_H */
